@@ -30,22 +30,22 @@
 #' dim_min <- 3 # minimum number of components
 #' dim_max <- 7 # maximum number of components
 #' nstart <- 10 # random starts for PARAFAC analysis, models built simulanuously, best selected
-#' cores <- parallel::detectCores()/2 # use all cores but do not use all threads
-#' maxit = 500
-#' ctol <- 10^-4 # tolerance for parafac
+#' cores <- parallel::detectCores(logical=FALSE) # use all cores but do not use all threads
+#' maxit = 1000
+#' ctol <- 10^-5 # tolerance for parafac
 #'
 #' pfres_comps <- eem_parafac(eem_list,comps=seq(dim_min,dim_max),
 #'     normalise = TRUE,maxit=10000,nstart=nstart,ctol=ctol,cores=cores)
 #' }
-eem_parafac <- function(eem_list,comps,maxit=500,normalise=TRUE,const=c("nonneg","nonneg","nonneg"),nstart = 10,ctol=10^-4,cores = parallel::detectCores()/2, verbose = FALSE, ...){
+eem_parafac <- function(eem_list, comps, maxit = 500, normalise = TRUE, const = c("nonneg","nonneg","nonneg"), nstart = 10, ctol = 10^-4, cores = parallel::detectCores(logical=FALSE), verbose = FALSE, ...){
   #eem_list <- eem_list %>% eem_red2smallest()
   eem_array <- eem2array(eem_list)
   if(normalise){
+    if(verbose) cat("EEM matrices are normalised!",fill=TRUE)
     eem_array <- eem_array %>% norm_array()
-    if(verbose) cat("EEM matrices were normalised!",fill=TRUE)
   }
+  if(verbose) cat(paste0(cores," cores are used for the calculation."),fill=TRUE)
   res <- lapply(comps,function(comp){
-    if(verbose) cat(paste0(cores," cores are used for the calculation."),fill=TRUE)
     if(verbose) cat(paste0("calculating ",comp," components model..."),fill=TRUE)
     #comp <- 5
     #eem_array %>% dim()
@@ -55,7 +55,7 @@ eem_parafac <- function(eem_list,comps,maxit=500,normalise=TRUE,const=c("nonneg"
     clusterExport(cl, c("eem_array","comp","maxit","nstart","const","ctol","cores"), envir=environment())
     clusterEvalQ(cl, library(multiway))
     }
-    cpresult <- parafac(eem_array,nfac=comp,const = const,maxit = maxit,parallel = (cores > 1), cl=cl, ctol=ctol, nstart = nstart,...)
+    cpresult <- parafac(eem_array, nfac = comp, const = const, maxit = maxit, parallel = (cores > 1), cl = cl, ctol = ctol, nstart = nstart, ...)
     if(cores > 1){
     stopCluster(cl)
     }
@@ -78,7 +78,7 @@ eem_parafac <- function(eem_list,comps,maxit=500,normalise=TRUE,const=c("nonneg"
 #' @description B and C modes (emission and excitation wavelengths) are rescaled to RMS of value newscale. This is compensated in A mode (sample loadings).
 #'
 #' @param pfmodel object of class parafac
-#' @param newscale Desired root mean-square for each column of rescaled mode. Can input a scalar or a vector with length equal to the number of factors for the given mode. If newscale = "Fmax", each component will be scaled so the maximum of each component is 1.
+#' @param newscale If (default) newscale = "Fmax", each component will be scaled so the maximum of each component is 1. It is also possible to set a desired root mean-square for each column of the rescaled mode. Can input a scalar or a vector with length equal to the number of factors for the given mode.
 #'
 #' @return object of class parafac
 #' @export
@@ -86,15 +86,15 @@ eem_parafac <- function(eem_list,comps,maxit=500,normalise=TRUE,const=c("nonneg"
 #' @seealso \code{\link[multiway]{rescale}}
 #'
 #' @examples
-#' data(pfres_comps1)
+#' data(pf_models)
 #'
-#' new_pf <- eempf_rescaleBC(pfres_comps[[2]])
-eempf_rescaleBC <- function(pfmodel,newscale = 1){
+#' new_pf <- eempf_rescaleBC(pf4[[1]])
+eempf_rescaleBC <- function(pfmodel,newscale = "Fmax"){
   nf <- attr(pfmodel,"norm_factors")
   comp <- ncol(pfmodel$A)
   if(newscale == "Fmax"){
-    Bmax <- pfmodel$B %>% matrixStats::colMaxs()
-    Cmax <- pfmodel$C %>% matrixStats::colMaxs()
+    Bmax <- pfmodel$B %>% abs() %>% matrixStats::colMaxs()
+    Cmax <- pfmodel$C %>% abs() %>% matrixStats::colMaxs()
     pfmodel$B <- pfmodel$B %*% diag(1/Bmax, nrow=comp) %>%
       `colnames<-`(colnames(pfmodel$B))
     pfmodel$C <- pfmodel$C %*% diag(1/Cmax, nrow=comp) %>%
@@ -113,6 +113,75 @@ eempf_rescaleBC <- function(pfmodel,newscale = 1){
   pfmodel
 }
 
+#' Extract names from PARAFAC model components
+#'
+#' @param pfmodel parafac model
+#'
+#' @return vector of names or list of vecters of names
+#' @export
+#'
+#' @examples
+#' data(pf_models)
+#' eempf_comp_names(pf4)
+#'
+#' eempf_comp_names(pf4) <- c("A","B","C","D","E","F","G")
+#'
+#' value <- list(c("A1","B1","C1","D","E","F","G"),
+#' c("A2","B2","C","D","E","F","G"),
+#' c("A3","B3","C","D","E","F","G"),
+#' c("A4","B4","C","D","E","F","G"),
+#' c("A5","B5","C","D","E","F","G5")
+#' )
+#'
+#' eempf_comp_names(pf4) <- value
+#' eempf_comp_names(pf4)
+#'
+#' ggeem(pf4[[1]])
+#'
+eempf_comp_names <- function(pfmodel){
+  if(class(pfmodel) == "parafac") {
+    colnames(pfmodel$A)
+    }else if(class(pfmodel) == "list" & class(pfmodel[[1]]) == "parafac"){
+      lapply(pfmodel, function(pfm) colnames(pfm$A))
+    } else{
+    stop("pfmodel is not a parafac model or a list of parafac models!")
+  }
+}
+
+#' Set names of PARAFAC components
+#'
+#' @param pfmodel model of class parafac
+#' @param value character vector containing the new names for the components
+#'
+#' @return parafac model
+#'
+#' @import dplyr
+#'
+#' @export
+#'
+#' @examples
+#' data(pf_models)
+#'
+#' eempf_comp_names(pf4) <- c("A","B","C","D","E","F","G")
+`eempf_comp_names<-` <- function(pfmodel, value){
+  if(class(pfmodel) == "parafac") {
+    colnames(pfmodel$A) <- value
+    colnames(pfmodel$B) <- value
+    colnames(pfmodel$C) <- value
+    pfmodel %>% `class<-`("parafac")
+  }else if(class(pfmodel) == "list" & class(pfmodel[[1]]) == "parafac"){
+    if(!is.list(value) | (length(value) == 1 & length(pfmodel) > 1)) value <- lapply(1:length(pfmodel), function(x) value)
+    lapply(1:length(pfmodel), function(pfn){
+      colnames(pfmodel[[pfn]]$A) <- value[[pfn]][1:ncol(pfmodel[[pfn]]$A)]
+      colnames(pfmodel[[pfn]]$B) <- value[[pfn]][1:ncol(pfmodel[[pfn]]$A)]
+      colnames(pfmodel[[pfn]]$C) <- value[[pfn]][1:ncol(pfmodel[[pfn]]$A)]
+      pfmodel[[pfn]] %>% `class<-`("parafac")
+    })
+  } else{
+    stop("pfmodel is not a parafac model or a list of parafac models!")
+  }
+}
+
 #' Data from an eemlist is transformed into an array
 #'
 #' @description Data matrices from EEM are combined to an array that is needed for a PARAFAC analysis.
@@ -129,20 +198,16 @@ eempf_rescaleBC <- function(pfmodel,newscale = 1){
 eem2array <- function(eem_list){
   eem_matrices <- eem_list %>%
     sapply("[", "x")
-
   dv <- lapply(eem_matrices,dim) %>% bind_cols()
   if(all(dv[1,1] %>% unlist() == dv[1,]) & all(dv[2,1] %>% unlist() == dv[2,])) dim_eem <- c(dv[,1] %>% unlist(), eem_matrices %>% length()) else dim_eem <- NA
   if(is.na(dim_eem[1])) stop("dimensions mismatch!")
 
   eem_array <- array(eem_matrices %>% unlist, dim=dim_eem)
-  ## set NA to 0
-  eem_array[is.na(eem_array) | is.nan(eem_array)] <- 0
   eem_array <- eem_array %>% aperm(perm = c(3,1,2), resize = TRUE)
   attr(eem_array,"em") <- eem_list[[1]]$em
   attr(eem_array,"ex") <- eem_list[[1]]$ex
   attr(eem_array,"samples") <- eem_list %>% sapply("[","sample") %>% unlist()
   attr(eem_array,"mdim") <- dim(eem_array)
-  #attr(eem_array,"norm_factors") <- NULL
   eem_array
 }
 
@@ -167,10 +232,6 @@ norm_array <- function(eem_array){
     sd(eem_array[s,,], na.rm=TRUE)
   }) %>% unlist()
   eem_array <- eem_array / norm_factors
-  #eem_array <- lapply(1:(dim(eem_array)[1]),function(s) {
-  #  eem_array[s,,]/norm_factors[s]
-  #}) %>% unlist() %>%
-  #  array(dim=dim(eem_array))
   attr(eem_array,"norm_factors") <- norm_factors
   eem_array
 }
@@ -191,18 +252,16 @@ norm_array <- function(eem_array){
 #' @import tidyr
 #'
 #' @examples
-#' data(pfres_comps1)
+#' data(pf_models)
 #'
-#' eempf_comp_mat(pfres_comps[[3]])
+#' eempf_comp_mat(pf4[[1]])
 eempf_comp_mat <- function(pfmodel,gather=TRUE){
-  #b[[3]] -> pfmodel
-  #comp <- 1
   mat <- lapply(seq(1:ncol(pfmodel$A)), function(comp){
     m <- matrix(pfmodel$B[,comp]) %*% t(matrix(pfmodel$C[,comp])) %>%
       data.frame()
     colnames(m) <- pfmodel$C %>% rownames()
     rownames(m) <- pfmodel$B %>% rownames()
-    if(gather==TRUE) m <- m %>% rownames_to_column("em") %>% gather(ex,value,-em)
+    if(gather==TRUE) m <- m %>% tibble::rownames_to_column("em") %>% gather(ex,value,-em)
     m
   })
   names(mat) <- colnames(pfmodel$B)
@@ -221,9 +280,9 @@ eempf_comp_mat <- function(pfmodel,gather=TRUE){
 #' @import dplyr
 #'
 #' @examples
-#' data(pfres_comps1)
+#' data(pf_models)
 #'
-#' eempf_leverage(pfres_comps[[2]])
+#' eempf_leverage(pf4[[1]])
 eempf_leverage <- function(pfmodel){
   cpl <- lapply(pfmodel[c("A","B","C")], function(M) diag(M %*% pinv(t(M) %*% M) %*% t(M)))
   names <- list(rownames(pfmodel$A),rownames(pfmodel$B),rownames(pfmodel$C))
@@ -245,14 +304,13 @@ eempf_leverage <- function(pfmodel){
 #' @importFrom stats ecdf
 #'
 #' @examples
-#' data(pfres_comps1)
+#' data(pf_models)
 #'
-#' eempf_mleverage(pfres_comps)
+#' eempf_mleverage(pf3)
 eempf_mleverage <- function(pfres_comps,ecdf = FALSE, stats = FALSE){
   cpls <- lapply(pfres_comps,eempf_leverage) %>%
     lapply(unlist) %>%
     lapply(function(ll) data.frame(parameter=names(ll),value=ll)) %>%
-    #    lapply(tibble::rownames_to_column,var="parameter") %>%
     list_join(by = "parameter") %>%
     `colnames<-`(c("parameter",paste0("comps",lapply(pfres_comps,function(cpout){ncol(cpout$A)}) %>% unlist())))
   if(ecdf){
@@ -281,9 +339,9 @@ eempf_mleverage <- function(pfres_comps,ecdf = FALSE, stats = FALSE){
 #' @importFrom stats quantile
 #'
 #' @examples
-#' data(pfres_comps1)
+#' data(pf_models)
 #'
-#' leverage <- eempf_leverage(pfres_comps[[2]])
+#' leverage <- eempf_leverage(pf4[[1]])
 #' lev_data <- eempf_leverage_data(leverage)
 eempf_leverage_data <- function(cpl,qlabel=0.1){
   cpl <- cpl %>%
@@ -310,9 +368,9 @@ eempf_leverage_data <- function(cpl,qlabel=0.1){
 #' @export
 #'
 #' @examples
-#' data(pfres_comps1)
+#' data(pf_models)
 #'
-#' pfres_comps[[2]][[3]] <- norm2A(pfres_comps[[2]][[3]])
+#' pf4[[1]] <- norm2A(pf4[[1]])
 norm2A <- function(pfmodel){
   if(!is.null(attr(pfmodel,"norm"))){
     pfmodel$A <- pfmodel$A * attr(pfmodel,"norm_factors")
@@ -335,8 +393,8 @@ norm2A <- function(pfmodel){
 #' @importFrom stats cor
 #'
 #' @examples
-#' data(pfres_comps1)
-#' eempf_cortable(pfres_comps[[2]])
+#' data(pf_models)
+#' eempf_cortable(pf4[[1]])
 eempf_cortable <- function(pfmodel,normalisation = FALSE, method="pearson",...){
   if(normalisation) pfmodel <- norm2A(pfmodel)
   pfmodel %>%
@@ -344,53 +402,9 @@ eempf_cortable <- function(pfmodel,normalisation = FALSE, method="pearson",...){
     cor(method=method,...)
 }
 
-
-#' Exclude complete wavelengths or samples form data set
+#' Extract data from emission and excitation wavelengths of the components of a PARAFAC model (scaled B- and C-modes)
 #'
-#' @description Outliers in all modes should be avoided. With this functions excitation or emission wavelengths as well as samples can be removed completely from your sample set.
-#'
-#' @param eem_list object of class eemlist
-#' @param exclude list of three vectors, see details
-#' @param verbose states whether additional information is given in the command line
-#'
-#' @details The argument exclude is a named list of three vectors. The names must be "ex", "em" and "sample". Each element contains a vector of wavelengths or sample names that are to be excluded from the data set.
-#'
-#' @return object of class eemlist
-#' @export
-#'
-#' @examples
-#' data(eem_list)
-#'
-#' exclude <- list("ex" = c(280,285,290,295),
-#' "em" = c(),
-#' "sample" = c("sample3","sample5","sample14")
-#' )
-#'
-#' eem_list_ex <- eem_exclude(eem_list, exclude)
-eem_exclude <- function(eem_list, exclude = list,verbose=FALSE){
-  ex_exclude <- exclude[["ex"]]
-  em_exclude <- exclude[["em"]]
-  sample_exclude <- exclude[["sample"]]
-  if(!is.null(sample_exclude)){
-    sample_exclude <- paste0("^",sample_exclude,"$")
-    eem_list <- eem_extract(eem_list,sample_exclude,verbose=verbose)
-  }
-  eem_list <- lapply(eem_list,function(eem){
-    #eem <- eem_list[[1]]
-    eem$x <- eem$x[!eem$em %in% em_exclude,!eem$ex %in% ex_exclude]
-    eem$ex <- eem$ex[!eem$ex %in% ex_exclude] #%>% length()
-    eem$em <- eem$em[!eem$em %in% em_exclude] #%>% length()
-    eem
-  })
-  if(!is.null(ex_exclude) & verbose) cat(paste0("Removed excitation wavelength(s): ",paste0(ex_exclude %>% sort(),collapse=", ")),fill=TRUE)
-  if(!is.null(em_exclude) & verbose) cat(paste0("Removed emission wavelength(s): ",paste0(em_exclude %>% sort(),collapse=", ")),fill=TRUE)
-  class(eem_list) = "eemlist"
-  eem_list
-}
-
-#' Extract data from emission and excitation wavelengths of the components of a PARAFAC model
-#'
-#' @description Data of wavelengths is returned. For each component the lines intersecting at the component maxima are returned.
+#' @description Data for each wavelengths is returned. For each component the lines intersecting at the component maxima are returned.
 #'
 #' @param pfmodel object of class parafac
 #'
@@ -401,13 +415,11 @@ eem_exclude <- function(eem_list, exclude = list,verbose=FALSE){
 #' @import tidyr
 #'
 #' @examples
-#' data(pfres_comps1)
+#' data(pf_models)
 #'
-#' ml <- maxlines(pfres_comps[[1]])
+#' ml <- maxlines(pf4[[1]])
 maxlines <- function(pfmodel){
-  #pfmodel <- pfres_comps[[2]]
   maxl <- lapply(colnames(pfmodel$C),function(comp){
-    #comp <- colnames(pfmodel$C)[1]
     em = (pfmodel$B[,comp]*max(pfmodel$C[,comp])) %>%
       data.frame(e="em", wavelength = as.numeric(names(.)),value=.)
     ex = (pfmodel$C[,comp]*max(pfmodel$B[,comp])) %>%
@@ -418,7 +430,6 @@ maxlines <- function(pfmodel){
     list_join(by=c("e","wavelength"))
 }
 
-
 #' Calculate residuals of EEM data according to a certain model
 #'
 #' @param pfmodel PARAFAC model of class parafac
@@ -437,19 +448,14 @@ maxlines <- function(pfmodel){
 #' @examples
 #' \donttest{
 #' data(eem_list)
-#' data(pfres_comps2)
+#' data(pf_models)
 #'
-#' eempf_residuals(pfres_comps2[[2]],eem_list)
+#' eempf_residuals(pf4[[1]],eem_list)
 #' }
 eempf_residuals <- function(pfmodel,eem_list,select=NULL, cores = parallel::detectCores(logical = FALSE)/2){
-  # pfmodel <- pfres_comps2[[which(comps==seq(dim_min,dim_max))]]
   pfmodel <- norm2A(pfmodel)
   if(!is.null(select)){
     eem_list <- eem_extract(eem_list,sample = select ,keep=TRUE,verbose = FALSE)
-      #lapply(select,function(s) which(s == eem_list %>% eem_names())) %>%
-      #unlist() %>%
-      #unique() %>%
-      #setdiff(seq(1,length(eem_list)),.)  %>% eem_extract(eem_list,.,verbose = FALSE)
   }
   if(!all(eem_names(eem_list) %in% rownames(pfmodel$A)) | length(eem_list) == 0){
     pfmodel <- A_missing(eem_list,pfmodel,cores=cores)
@@ -487,16 +493,22 @@ eempf_residuals <- function(pfmodel,eem_list,select=NULL, cores = parallel::dete
 
 #' Calculate the amount of each component for samples not involved in model building
 #'
-#' Samples from an eemlist that were not used in the modelling process are added as entries in the  A-modes. Values are calculated using fixed B and C modes in the PARAFAC algorithm.
+#' @description Samples from an eemlist that were not used in the modelling process are added as entries in the  A-modes. Values are calculated using fixed B and C modes in the PARAFAC algorithm. B and C modes can be provided via a previously calculated model or as matrices manually.
 #'
 #' @param eem_list object of class eemlist with sample data
 #' @param pfmodel object of class parafac
 #' @param cores number of cores to use for parallel processing
+#' @param components optionally supply components to use manually, either as a variable of class parafac_components or as a list of variables of class parafac_components, if you do so,
+#' @param const optional constraints for model, just used, when components are supplied
+#' @param control optional constraint control parameters for model, just used, when components are supplied
 #' @param ... additional arguments passed to eem_parafac
 #'
+#' @details This function can be used to calculate A modes (sample loadings) for samples that were previously excluded from the modelling process (e.g. outliers). Another way to use it would be a recombination of components from different models and calculating the according sample loadings. Expecially the later application is experimental and results have to be seen critically! Nevertheless, I decided to supply this function to stimulate some experiments on that and would be interested in your findings and feedback.
+#'
 #' @return object of class parafac
 #' @export
 #'
+#' @import dplyr
 #' @import parallel
 #' @import foreach
 #' @importFrom doParallel registerDoParallel
@@ -504,28 +516,40 @@ eempf_residuals <- function(pfmodel,eem_list,select=NULL, cores = parallel::dete
 #' @examples
 #' \donttest{
 #' data(eem_list)
-#' data(pfres_comps2)
+#' data(pf_models)
 #'
-#' A_missing(eem_list,pfres_comps2[[3]])
+#' A_missing(eem_list,pf4[[1]])
 #' }
-A_missing <- function(eem_list,pfmodel,cores = parallel::detectCores(logical = FALSE)/2,...){
-  #data("pfres_comps2")
-  #data("eem_list")
-  #cores=2
-  #pfmodel <- pfres_comps2[[2]]
-
+A_missing <- function(eem_list,pfmodel = NULL,cores = parallel::detectCores(logical = FALSE),components = NULL, const = NULL, control = NULL, ...){
   eem_list <- eem_red2smallest(eem_list)
+  if(is.null(pfmodel) & is.null(components)) stop("You must either specify a model or components as a base for the newly generated model!")
 
   exclude <- list("ex" = eem_list[[1]]$ex[!(eem_list[[1]]$ex %in% rownames(pfmodel$C))],
                   "em" = eem_list[[1]]$em[!(eem_list[[1]]$em %in% rownames(pfmodel$B))],
                   "sample" = c()
   )
-
   x <- eem_list %>%
     eem_exclude(exclude)
-
-  missingAs <- eem_parafac(x,comps = pfmodel$A %>% ncol(),normalise = (!is.null(attr(pfmodel,"norm_factors"))),Bfixed = pfmodel$B, Cfixed = pfmodel$C,cores = cores,control = pfmodel$control, const = pfmodel$const,...)
-
+  if(!is.null(components)){
+    if(!is.null(pfmodel)) warning("The base model is ignored since you provided components manually!")
+    if(class(components[[1]]) == "parafac_components") components <- eempf_bindxc(components)
+    if(class(components) == "parafac_components"){
+      Bfixed <- components$B
+      Cfixed <- components$C
+      comps <- ncol(components$B)
+      if(is.null(const)) const <- c("nonneg", "nonneg", "nonneg")
+    } else {
+      stop("The list of components you supplied is invalid!")
+    }
+  } else {
+    Bfixed <- pfmodel$B
+    Cfixed <- pfmodel$C
+    comps <- pfmodel$A %>% ncol()
+    normalise = (!is.null(attr(pfmodel,"norm_factors")))
+    control = pfmodel$control
+    const = pfmodel$const
+  }
+  missingAs <- eem_parafac(x,comps = comps,normalise = (!is.null(attr(pfmodel,"norm_factors"))),Bfixed = Bfixed, Cfixed = Cfixed,cores = cores,const = const, control = control, ...)
   missingAs[[1]]
 }
 
@@ -542,6 +566,7 @@ A_missing <- function(eem_list,pfmodel,cores = parallel::detectCores(logical = F
 #' @param cores number of parallel calculations (e.g. number of physical cores in CPU)
 #' @param maxit maximum iterations for PARAFAC algorithm
 #' @param ctol Convergence tolerance (R^2 change)
+#' @param rescale rescale splithalf models to Fmax, see \code{\link[staRdom]{eempf_rescaleBC}}
 #' @param verbose states whether you want additional information during calculation
 #' @param ... additional parameters that are passed on to \code{\link[multiway]{parafac}}
 #'
@@ -560,9 +585,10 @@ A_missing <- function(eem_list,pfmodel,cores = parallel::detectCores(logical = F
 #' \donttest{
 #' data(eem_list)
 #'
-#' splithalf(eem_list,6,nstart=2)
+#' splithalf <- splithalf(eem_list,6,nstart=20)
+#' splithalf_plot(splithalf)
 #' }
-splithalf <- function(eem_list,comps,splits=NA,rand=FALSE,normalise=TRUE,nstart=10,cores=parallel::detectCores()/2,maxit=500,ctol = 10^(-5),...,verbose =FALSE){
+splithalf <- function(eem_list, comps, splits = NA, rand = FALSE, normalise = TRUE, nstart = 10, cores = parallel::detectCores(logical = FALSE), maxit = 1000, ctol = 10^(-5), rescale = TRUE, verbose = FALSE, ...){
   a <- seq(1,eem_list %>% length())
   if(rand){
     a <- a %>% sample()
@@ -573,13 +599,23 @@ splithalf <- function(eem_list,comps,splits=NA,rand=FALSE,normalise=TRUE,nstart=
     eem_list %>% eem_extract(splits[co] %>% unlist() %>% sort(),keep=TRUE, verbose=FALSE)
 
   })
-  if(verbose) cat(paste0(cores," cores are used for the calculation."),fill=TRUE)
-  fits <- lapply(spl_eems,function(eem){eem_parafac(eem,comps=comps,normalise = normalise,maxit=maxit,nstart = nstart, cores = cores,ctol = ctol,verbose=verbose,...)}) #
-
+  if(verbose){
+    cat(paste0(cores," cores are used for the calculation."),fill=TRUE)
+    if(normalise){
+      if(verbose) cat("EEM matrices are normalised!",fill=TRUE)
+    }
+    cat(paste0("Calculating PARAFAC models with split-half data..."),fill=TRUE)
+    pb <- txtProgressBar(max = 6, style=3)
+  }
+  fits <- lapply(1:length(spl_eems),function(i){
+    mod <- eem_parafac(spl_eems[[i]],comps=comps,normalise = normalise,maxit=maxit,nstart = nstart, cores = cores,ctol = ctol,verbose=FALSE,...)
+    if(rescale) mod <- lapply(mod,eempf_rescaleBC,newscale="Fmax")
+    if(verbose) setTxtProgressBar(pb, i)
+    mod
+    }) #
+  if(verbose) close(pb)
   reallign <- tcc_find_pairs(fits)
 
-  sel <- 4
-
   C_sort <- lapply(2:6,function(sel){
     fit <- fits[[sel]]
     reallign %>%
@@ -664,7 +700,7 @@ tcc_find_pairs <- function(fits){
     comb_nam <- colnames(t) %>% .[. != e]
     t <- t %>%
       select(one_of(comb_nam)) %>%
-      congru()
+      multiway::congru()
     rownames(t) <- comb_nam
     colnames(t) <- comb_nam
     attr(t,"e") <- e
@@ -674,7 +710,7 @@ tcc_find_pairs <- function(fits){
 
   arr <- ct %>%
     .[["em"]] %>%
-    .[1:comps,(comps+1):(6*comps)] %>%
+    .[1:comps,(comps+1):(length(fits)*comps)] %>%
     data.frame()
 
   a <- data.frame("comp1" = arr %>% rownames(),stringsAsFactors = FALSE)
@@ -701,9 +737,7 @@ tcc_find_pairs <- function(fits){
     mutate(tcc_ex = ct[[1]][comp1,paste0(selection,comp2)],tcc_em = ct[[2]][comp1,paste0(selection,comp2)]) %>%
     ungroup()
 
-
-  split_designations <- c("AB","AC","AD","BC","BD","CD")
-
+  if(length(fits) == 6){
   tt <- arrange_emex %>%
     select(comp1) %>%
     distinct(comp1) %>%
@@ -714,6 +748,8 @@ tcc_find_pairs <- function(fits){
     spread(selection,comp2) %>%
     ungroup()
 
+  split_designations <- c("AB","AC","AD","BC","BD","CD")
+
   ttt <- lapply(1:3,function(i){
     tt %>%
       select(comp1,one_of(paste0(i),paste0(7-i))) %>%
@@ -727,6 +763,7 @@ tcc_find_pairs <- function(fits){
     arrange(component) %>%
     select(component,comb,tcc_ex,tcc_em)
   attr(arrange_emex,"tcc_table") <- ttt
+  }
   arrange_emex
 }
 
@@ -763,7 +800,7 @@ splithalf_splits <- function(fits){
 
 #' Caluclate Tucker's Congruence Coefficient of PARAFAC components
 #'
-#' @description Componets must be passed as peak lines \code{\link[staRdom]{maxlines}}
+#' @description Componets must be passed as modes, see \code{\link[staRdom]{maxlines}}
 #'
 #' @param maxl_table data frame containing the peak lines of components
 #' @param na.action if "na.omit" NA are deleted from prior the test
@@ -778,18 +815,15 @@ splithalf_splits <- function(fits){
 #' @importFrom stats na.omit
 #'
 #' @examples
-#' data(pfres_comps1)
+#' data(pf_models)
 #'
-#' ml <- maxlines(pfres_comps[[2]])
+#' ml <- maxlines(pf4[[1]])
 #'
 #' tcc(ml)
 tcc <- function(maxl_table,na.action="na.omit"){
-  #maxl_table <- maxl %>% full_join(imp_table,by=c("e","wavelength"))
-  #E="ex"
   c <- lapply(c("em","ex"), function(E) {
     c2 <- maxl_table %>% filter(e == E) %>% select(-e) %>% arrange(wavelength)
     if(na.action == "na.omit") c2 <- c2 %>% na.omit()
-    #if(na.action == "approx") c2 <- c2 %>% zoo::na.approx(xout = .$wavelength, na.rm = FALSE) %>% data.frame() %>% na.omit()
     c2 <- c2 %>%
       select(-wavelength) %>%
       congru()
@@ -816,8 +850,8 @@ tcc <- function(maxl_table,na.action="na.omit"){
 #' @importFrom stringr str_replace_all
 #'
 #' @examples
-#'   data(pfres_comps2)
-#'   eempf_openfluor(pfres_comps2[[2]],file.path(tempdir(),"openfluor_example.txt"))
+#'   data(pf_models)
+#'   eempf_openfluor(pf4[[1]],file.path(tempdir(),"openfluor_example.txt"))
 eempf_openfluor <- function(pfmodel,file){
   if(!dir.exists(dirname(file.path(file)))){
     stop("The path to your file does not contain an existing directory. Please enter a correct path!")
@@ -829,7 +863,7 @@ eempf_openfluor <- function(pfmodel,file){
   template <- system.file("openfluor_template.txt",package="staRdom")
   template <- readLines(template)
   template <- stringr::str_replace_all(template,"toolbox","toolbox\tstaRdom")
-  template <- stringr::str_replace_all(template,"nSample",paste0("nSample\t",pfmodel$C %>% nrow()))
+  template <- stringr::str_replace_all(template,"nSample",paste0("nSample\t",pfmodel$A %>% nrow()))
   write(template,file)
   write.table(factors,file=file,append=TRUE,col.names=FALSE,row.names=FALSE,sep="\t",quote=FALSE)
   message("An openfluor file has been successfully written. Please fill in missing header fields manually!")
@@ -859,11 +893,10 @@ eempf_openfluor <- function(pfmodel,file){
 #' @examples
 #' \donttest{
 #' data(eem_list)
-#' data(pfres_comps2)
-#' data(abs_data)
+#' data(pf_models)
 #'
-#' results <- eempf4analysis(pfmodel = pfres_comps2[[2]],
-#'                           eem_list = eem_list, absorbance = abs_data,
+#' results <- eempf4analysis(pfmodel = pf4[[1]],
+#'                           eem_list = eem_list,
 #'                           cuvl = 5, n = 4)
 #'                           }
 eempf4analysis <- function(pfmodel,eem_list = NULL, absorbance = NULL, cuvl = NULL, n = 4, export = NULL,...){
@@ -910,9 +943,9 @@ eempf4analysis <- function(pfmodel,eem_list = NULL, absorbance = NULL, cuvl = NU
 #' @importFrom utils write.table
 #'
 #' @examples
-#' data(pfres_comps2)
+#' data(pf_models)
 #'
-#' factor_table <- eempf_export(pfres_comps2[[2]])
+#' factor_table <- eempf_export(pf4[[1]])
 eempf_export<- function(pfmodel,export = NULL,...){
   tabs <- list(pfmodel$A %>%
                  as.data.frame() %>%
@@ -958,13 +991,13 @@ eempf_export<- function(pfmodel,export = NULL,...){
 #' \dontrun{
 #' # due to data limitation in package, example does not work with that data!
 #'
-#' eempf_corecondia(pfmodel,eem_list)
+#' # eempf_corcondia(pfmodel,eem_list)
 #'
 #' }
 eempf_corcondia <- function(pfmodel,eem_list,divisor="core"){
   arr <- eem_list %>% eem2array()
   if(!is.null(attr(eem_list,"norm_factors"))) arr <- arr %>% norm_array()
-  multiway::corcondia(arr,pfmodel,divisor="core")
+  multiway::corcondia(arr,pfmodel,divisor=divisor)
 }
 
 #' Calculating EEMqual which is an indicator of a PARAFAC model's quality
@@ -986,10 +1019,10 @@ eempf_corcondia <- function(pfmodel,eem_list,divisor="core"){
 #' @examples
 #' \donttest{
 #' data(eem_list)
-#' data(pfres_comps2)
+#' data(pf_models)
 #'
-#' pfmodel <- pfres_comps2[[2]]
-#' eempf_eemqual(eem_list,pfmodel)
+#' pfmodel <- pf4[[1]]
+#' eempf_eemqual(eem_list,pfmodel) # insuficient example data to run!
 #' }
 eempf_eemqual <- function(pfmodel,eem_list,splithalf = NULL, ...){
   corec <- eempf_corcondia(pfmodel,eem_list)
@@ -1004,3 +1037,123 @@ eempf_eemqual <- function(pfmodel,eem_list,splithalf = NULL, ...){
   attr(tab,"shmodel") <- sh
   tab
 }
+
+#' Calculate the importance of each component.
+#'
+#' @param pfmodel model of class parafac
+#' @param eem_list eemlist used to calculate that model
+#' @param cores cores to be used for the calculation
+#' @param ... other aruments passed to eem_parafac
+#'
+#' @details The importance of each variable is calculated by means of creating a model without a specific component and calculating the difference between the original R-squared and the one with the left out component. The derived values state the loss in model fit if one component is not used in the modeling process. For the creation of the new models, the exact components of the original model are used.
+#'
+#' @return numeric vector, values are in the same order of the components in the supplied model.
+#' @export
+#'
+#' @import dplyr
+#' @import tidyr
+#'
+#' @examples
+#' \donttest{
+#' data(pfmodel)
+#' data(eem_list)
+#'
+#' eempf_varimp(pf4[[1]],eem_list)
+#' }
+eempf_varimp <- function(pfmodel,eem_list, cores = parallel::detectCores(logical=FALSE),...){
+
+  exclude <- list("ex" = eem_list[[1]]$ex[!(eem_list[[1]]$ex %in% rownames(pfmodel$C))],
+                  "em" = eem_list[[1]]$em[!(eem_list[[1]]$em %in% rownames(pfmodel$B))],
+                  "sample" = c()
+  )
+  x <- eem_list %>%
+    eem_red2smallest() %>%
+    eem_exclude(exclude)
+  mods <- lapply(1:(pfmodel$A %>% ncol()), function(c){
+    eem_parafac(x,comps = ncol(pfmodel$A)-1,normalise = (!is.null(attr(pfmodel,"norm_factors"))),Bfixed = pfmodel$B[,-c], Cfixed = pfmodel$C[,-c],cores = cores,control = pfmodel$control, const = pfmodel$const)
+  })
+  Rsq_al <- pfmodel$Rsq - (mods %>% lapply(function(mod) {mod[[1]]$Rsq}) %>% unlist())
+}
+
+
+#' Reorder PARAFAC components
+#'
+#' @param pfmodel model of class parafac
+#' @param order vector containing desired new order or "em" or "ex" to reorder according to emission or excitation wavelengths of the peaks
+#' @param decreasing logical, whether components are reordered according to peak wvalengths in a decreasing direction
+#'
+#' @return parafac model
+#'
+#' @import dplyr
+#' @importFrom multiway reorder.parafac
+#'
+#' @export
+#'
+#' @examples
+#' data(pf_models)
+#' ggeem(pf4[[1]])
+#'
+#' pf4r <- eempf_reorder(pf4[[1]],"ex")
+#' ggeem(pf4r)
+eempf_reorder <- function(pfmodel,order,decreasing = FALSE){
+  if(!(order == "em" | order == "ex" | is.vector(order))) stop("no valid data suppli ed for order!")
+  if(order == "em") order <- apply(pfmodel$B,2,which.max) %>% sort.list(decreasing = decreasing)
+  if(order == "ex") order <- apply(pfmodel$C,2,which.max) %>% sort.list(decreasing = decreasing)
+  if(ncol(pfmodel$A) != length(order)) stop("the length of the order vector does not fit the number of components")
+
+  mod <- try(multiway::reorder.parafac(pfmodel,neworder = order), silent=TRUE)
+  if(class(mod) =="try-error") stop(mod) else {
+    mod
+  }
+}
+
+
+#' Extracting components of a PARAFAC model
+#'
+#' @param pfmodel parafac model
+#' @param comps vector with numbers of components to extract
+#'
+#' @return list
+#' @export
+#'
+#' @import dplyr
+#'
+#' @examples
+#' data(pf_models)
+#' pfmodel <- pf4[[1]]
+#' comps <- eempf_excomp(pfmodel,c(1,3))
+eempf_excomp <- function(pfmodel,comps){
+  list(pfmodel$B[,comps], pfmodel$C[,comps]) %>%
+    `names<-`(c("B","C")) %>%
+    `class<-`("parafac_components")
+}
+
+
+#' Combining extracted components of PARAFAC models
+#'
+#' @param components list of parafac_components
+#'
+#' @return parafac_components
+#' @export
+#'
+#' @import dplyr
+#'
+#' @examples
+#'
+#' data(pf_models)
+#' pfmodel <- pf4[[1]]
+#' comps <- eempf_excomp(pfmodel,c(1,3))
+#' comps2 <- eempf_excomp(pfmodel,c(4,6))
+#' comps3 <- eempf_bindxc(list(comps, comps2))
+#'
+eempf_bindxc <- function(components){
+B <- lapply(components, `[[`, "B") %>%
+  do.call(cbind,.)
+C <- lapply(components, `[[`, "C") %>%
+  do.call(cbind,.)
+  list(B,C) %>%
+  `names<-`(c("B","C")) %>%
+  `class<-`("parafac_components")
+}
+
+
