@@ -1,16 +1,24 @@
 ##
 #' EEM spectra plotted with ggplot2
 #'
-#' @description \code{ggeem} creates nice plots from EEM spectra of class ggplot. Plots can be modified as any ggplot by adding layers and/or elements with "+".
+#' @description Plots from EEM spectra of class \code{ggplot}. In case you work with a larger number of EEMs and want to show then in several plots, you can use \code{\link{eem_overview_plot}}.
 #'
-#' @param data eem, eemlist, parafac or data.frame. The details are given under 'Details'
-#' @param fill_max FALSE
-#' @param ... parameters passed on to \code{ggplot}
+#' @param data eem, eemlist, parafac or data.frame. The details are given under 'Details'.
+#' @param fill_max sets the maximum fluorescence value for the colour scale. This is mainly used by other functions, and makes different plots visually comparable.
+#' @param redneg logical, whether negative values should be coloured discreet.
+#' @param ... parameters passed on to \code{ggplot}.
 #'
 #' @details The data can be of different sources:
-#'     eem: a single EEM pectrum is plotted
-#'     eemlist: all spectra of the samples are plotted in one facet plot
+#'     eem: a single EEM spectrum is plotted
+#'     eemlist: all spectra of the samples are plotted, arranged in a grid
 #'     data.frame: a data.frame containing EEM data. Can be created by e.g. \code{as.data.frame.eem}
+#'     parafac: a PARAFAC model, the components are plotted then.
+#'
+#'     Using redneg you can give negative values a reddish colour. This can help identifying these parts in samples or components. Negative values are physically not possible and can only be the result of measuring errors, model deviations and problems with interpolated values.
+#'
+#'     A colour palette can be specified using the argument colpal.
+#'
+#'     Plotting distinct samples can be done using \code{\link{eem_extract}}. Please see example.
 #'
 #' @return a ggplot object
 #'
@@ -19,9 +27,10 @@
 #' @importFrom grDevices rainbow
 #'
 #' @examples
-#' ## plotting one distinct sample
+#' ## plotting two distinct samples
 #' data(eem_list)
-#' eem <- eem_extract(eem_list,c("sample6","sample7"),keep=TRUE)
+#' eem_names(eem_list)
+#' eem <- eem_extract(eem_list,c("^dreem_667sf$", "^dreem_661sf$"),keep=TRUE)
 #' ggeem(eem)
 ggeem <- function(data, fill_max=FALSE, ...) UseMethod("ggeem")
 
@@ -37,7 +46,7 @@ ggeem.eemlist <- function(data,fill_max=FALSE,...)
 {
   table <- data %>% lapply(as.data.frame) %>% bind_rows()
   #filename <- paste0('EEM_spectra_',suffix,format(Sys.time(), "%Y%m%d_%H%M%S"))
-  ggeem(table,fill_max=fill_max)
+  ggeem(table,fill_max=fill_max,...)
 }
 
 #' @rdname ggeem
@@ -53,18 +62,23 @@ ggeem.eem <- function(data,fill_max=FALSE,...)
 #' @export
 ggeem.parafac <- function(data,fill_max=FALSE,...)
 {
-  table <- data  %>% eempf_comp_mat() #eem_list
+  table <- data %>% eempf_comp_mat() #eem_list
   table <- lapply(table %>% names(),function(name){
     table[[name]] %>% mutate(sample = name)
-  }) %>% bind_rows()
+  }) %>% bind_rows() %>%
+    mutate(sample = factor(sample, levels = colnames(data$A)))
   #filename <- paste0('EEM_PARAFAC_components_',suffix,format(Sys.time(), "%Y%m%d_%H%M%S"))
   ggeem(table,fill_max=fill_max,...)
 }
 
 #' @rdname ggeem
 #' @export
-ggeem.data.frame <- function(data,fill_max=FALSE,...)
+ggeem.data.frame <- function(data,fill_max=FALSE,redneg = FALSE, ...)
 {
+  if(!exists("colpal")){
+    colpal <- rainbow
+    # warning("using rainbow colour palette")
+  }
   table <- data
   breaksx <- table %>% select(ex) %>% unique() %>% filter(as.numeric(ex)%%50 == 0) %>% unlist()
   breaksy <- table %>% select(em) %>% unique() %>% filter(as.numeric(em)%%50 == 0) %>% unlist()
@@ -72,9 +86,8 @@ ggeem.data.frame <- function(data,fill_max=FALSE,...)
     fill_max <- table$value %>% max(na.rm=TRUE)
   }
   #values_fill <- seq(0,fill_max,length.out = 55)
-  plot <- table %>% ggplot(...)+
+  plot <- table %>% ggplot()+
     geom_raster(aes(x=ex,y=em,fill=value))+ #,interpolate=TRUE
-    scale_fill_gradientn(colours=rainbow(75)[51:1],limits = c(0,fill_max))+
     #scale_fill_gradient2(low="blue",mid="yellow",high="red")
     scale_x_discrete(breaks = breaksx) +
     scale_y_discrete(breaks = breaksy) +
@@ -83,8 +96,16 @@ ggeem.data.frame <- function(data,fill_max=FALSE,...)
   if(table$value %>% min(na.rm=TRUE) < 0){
     vals <- c(table$value %>% min(na.rm=TRUE),seq(from=0,to=fill_max,length.out = 51))
     vals <- (vals - min(vals))/diff(range(vals))
+    if(redneg){
+      plot <- plot +
+        scale_fill_gradientn(colours=c(colpal(75)[58],colpal(75)[51:1]),values=vals,limits = c(table$value %>% min(na.rm=TRUE),fill_max))
+    } else {
+      plot <- plot +
+        scale_fill_gradientn(colours=colpal(75)[52:1],values=vals,limits = c(table$value %>% min(na.rm=TRUE),fill_max))
+    }
+  } else {
     plot <- plot +
-      scale_fill_gradientn(colours=c(rainbow(75)[58],rainbow(75)[51:1]),values=vals,limits = c(table$value %>% min(na.rm=TRUE),fill_max))
+      scale_fill_gradientn(colours=colpal(75)[51:1],limits = c(0,fill_max))
   }
   plot
 }
@@ -94,6 +115,7 @@ ggeem.data.frame <- function(data,fill_max=FALSE,...)
 #'
 #' @param data fluorescence data of class eemlist
 #' @param spp number of samples per plot
+#' @param ... arguments passed on to \code{\link[staRdom]{ggeem}}
 #'
 #' @return list of ggplots
 #' @export
@@ -101,18 +123,18 @@ ggeem.data.frame <- function(data,fill_max=FALSE,...)
 #' @examples
 #' \donttest{
 #' data(eem_list)
-#' eem_overview_plot(eem_list,spp=3)
+#' eem_overview_plot(eem_list,spp=9)
 #' }
-eem_overview_plot <- function(data,spp = 8){
+eem_overview_plot <- function(data,spp = 8,...){
   #data <- eem_list
   ppp <- data %>% length()/spp
   fill_max <- data %>% eem_scale_ext() %>% .[2]
   #print(fill_max)
   ov_plot <- lapply(1:ceiling(ppp),function(pos){
-    #pos <- 2
+    #pos <- 1
     data[(spp*(pos-1)+1):(spp*pos)] %>%
       `attr<-`("class", "eemlist") %>%
-      ggeem(fill_max=fill_max)
+      ggeem(fill_max=fill_max,...)#,...
   })
   ov_plot
 }
